@@ -391,7 +391,7 @@ CREATE TABLE t_ds_task_definition (
   PRIMARY KEY (id)
 ) ;
 
-create index task_definition_index on t_ds_task_definition (project_code,id);
+create index task_definition_index on t_ds_task_definition (code,id);
 
 DROP TABLE IF EXISTS t_ds_task_definition_log;
 CREATE TABLE t_ds_task_definition_log (
@@ -422,6 +422,9 @@ CREATE TABLE t_ds_task_definition_log (
   PRIMARY KEY (id)
 ) ;
 
+create index idx_code_version on t_ds_task_definition_log (code,version);
+create index idx_task_definition_log_project_code on t_ds_task_definition_log (project_code);
+
 DROP TABLE IF EXISTS t_ds_process_task_relation;
 CREATE TABLE t_ds_process_task_relation (
   id serial NOT NULL  ,
@@ -440,7 +443,7 @@ CREATE TABLE t_ds_process_task_relation (
   PRIMARY KEY (id)
 ) ;
 
-create index project_code_process_definition_code_index on t_ds_process_task_relation (project_code,process_definition_code);
+create index idx_code on t_ds_process_task_relation (project_code,process_definition_code);
 
 DROP TABLE IF EXISTS t_ds_process_task_relation_log;
 CREATE TABLE t_ds_process_task_relation_log (
@@ -461,7 +464,7 @@ CREATE TABLE t_ds_process_task_relation_log (
   update_time timestamp DEFAULT NULL ,
   PRIMARY KEY (id)
 ) ;
-
+create index idx_process_code_version on t_ds_process_task_relation_log (process_definition_code,process_definition_version);
 --
 -- Table structure for table t_ds_process_instance
 --
@@ -637,8 +640,8 @@ CREATE TABLE t_ds_resources (
   create_time timestamp DEFAULT NULL ,
   update_time timestamp DEFAULT NULL ,
   pid int,
-  full_name varchar(64),
-  is_directory int,
+  full_name varchar(128),
+  is_directory boolean DEFAULT FALSE,
   PRIMARY KEY (id),
   CONSTRAINT t_ds_resources_un UNIQUE (full_name, type)
 ) ;
@@ -721,6 +724,8 @@ CREATE TABLE t_ds_task_instance (
   PRIMARY KEY (id)
 ) ;
 
+create index idx_task_instance_code_version on t_ds_task_instance (task_code, task_definition_version);
+
 --
 -- Table structure for table t_ds_tenant
 --
@@ -928,7 +933,7 @@ INSERT INTO t_ds_queue(queue_name, queue, create_time, update_time)
 VALUES ('default', 'default', '2018-11-29 10:22:33', '2018-11-29 10:22:33');
 
 -- Records of t_ds_queue,default queue name : default
-INSERT INTO t_ds_version(version) VALUES ('2.0.2');
+INSERT INTO t_ds_version(version) VALUES ('2.0.6');
 
 --
 -- Table structure for table t_ds_plugin_define
